@@ -10,7 +10,7 @@ commonmeta-py is a Python library to convert scholarly metadata
 """
 
 __title__ = "commonmeta-py"
-__version__ = "0.133"
+__version__ = "0.174"
 __author__ = "Martin Fenner"
 __license__ = "MIT"
 
@@ -25,9 +25,14 @@ from .author_utils import (
 )
 from .base_utils import (
     compact,
+    dig,
+    get_crossref_xml_head,
     parse_attributes,
+    parse_xml,
     presence,
     sanitize,
+    unparse_xml,
+    unparse_xml_list,
     unwrap,
     wrap,
 )
@@ -77,6 +82,7 @@ from .readers import (
     ris_reader,
     schema_org_reader,
 )
+from .schema_utils import json_schema_errors, xml_schema_errors
 from .utils import (
     dict_to_spdx,
     extract_curie,
@@ -112,3 +118,20 @@ from .writers import (
     ris_writer,
     schema_org_writer,
 )
+from .writers.crossref_xml_writer import (
+    MARSHMALLOW_MAP,
+    CrossrefBadRequestError,
+    CrossrefError,
+    CrossrefForbiddenError,
+    CrossrefGoneError,
+    CrossrefNoContentError,
+    CrossrefNotFoundError,
+    CrossrefRequestError,
+    CrossrefServerError,
+    CrossrefUnauthorizedError,
+    CrossrefXMLClient,
+    CrossrefXMLSchema,
+    HttpError,
+    convert_crossref_xml,
+    write_crossref_xml,
+)
