@@ -73,6 +73,12 @@ def convert(
 @click.option("--depositor", type=str)
 @click.option("--email", type=str)
 @click.option("--registrant", type=str)
+@click.option("--login_id", type=str)
+@click.option("--login_passwd", type=str)
+@click.option("--test_mode", type=bool, default=False)
+@click.option("--host", type=str)
+@click.option("--token", type=str)
+@click.option("--legacy-key", type=str)
 @click.option("--show-errors/--no-errors", type=bool, show_default=True, default=False)
 def put(
     input,
@@ -85,23 +91,34 @@ def put(
     depositor,
     email,
     registrant,
+    login_id,
+    login_passwd,
+    test_mode,
+    host,
+    token,
+    legacy_key,
     show_errors,
 ):
-    metadata = Metadata(input, via=via, doi=doi, prefix=prefix)
+    metadata = Metadata(
+        input,
+        via=via,
+        doi=doi,
+        depositor=depositor,
+        email=email,
+        registrant=registrant,
+        login_id=login_id,
+        login_passwd=login_passwd,
+        test_mode=test_mode,
+        host=host,
+        token=token,
+        legacy_key=legacy_key,
+        prefix=prefix,
+    )
     if show_errors and not metadata.is_valid:
         raise click.ClickException(str(metadata.errors) + str(metadata.write_errors))
 
-    click.echo(
-        metadata.write(
-            to=to,
-            style=style,
-            locale=locale,
-            depositor=depositor,
-            email=email,
-            registrant=registrant,
-        )
-    )
-    if show_errors and metadata.write_errors:
+    click.echo(metadata.push(to=to, style=style, locale=locale))
+    if show_errors and len(metadata.write_errors) > 0:
         raise click.ClickException(str(metadata.write_errors))
 
 
@@ -169,6 +186,7 @@ def list(
 @click.option("--registrant", type=str)
 @click.option("--login_id", type=str)
 @click.option("--login_passwd", type=str)
+@click.option("--test_mode", type=bool, default=False)
 @click.option("--host", type=str)
 @click.option("--token", type=str)
 @click.option("--legacy-key", type=str)
@@ -187,6 +205,7 @@ def push(
     registrant,
     login_id,
     login_passwd,
+    test_mode,
     host,
     token,
     legacy_key,
@@ -204,6 +223,7 @@ def push(
         registrant=registrant,
         login_id=login_id,
         login_passwd=login_passwd,
+        test_mode=test_mode,
         host=host,
         token=token,
         legacy_key=legacy_key,
