@@ -240,36 +240,53 @@ DC_TO_CM_TRANSLATIONS = {
 
 # https://github.com/zenodo/zenodo/blob/master/zenodo/modules/records/data/objecttypes.json
 INVENIORDM_TO_CM_TRANSLATIONS = {
-    "book": "Book",
-    "section": "BookChapter",
-    "conferencepaper": "ProceedingsArticle",
-    "patent": "Patent",
-    "publication": "JournalArticle",
-    "publication-blogpost": "BlogPost",
-    "publication-preprint": "Article",
-    "report": "Report",
-    "softwaredocumentation": "Software",
-    "thesis": "Dissertation",
-    "technicalnote": "Report",
-    "workingpaper": "Report",
-    "datamanagementplan": "OutputManagementPlan",
     "annotationcollection": "Collection",
-    "taxonomictreatment": "Collection",
+    "book": "Book",
+    "conferencepaper": "ProceedingsArticle",
+    "datamanagementplan": "OutputManagementPlan",
+    "dataset": "Dataset",
+    "drawing": "Image",
+    "figure": "Image",
+    "image": "Image",
+    "lesson": "InteractiveResource",
+    "other": "Other",
+    "patent": "Patent",
     "peerreview": "PeerReview",
+    "photo": "Image",
+    "physicalobject": "PhysicalObject",
+    "plot": "Image",
     "poster": "Presentation",
     "presentation": "Presentation",
-    "dataset": "Dataset",
-    "figure": "Image",
-    "plot": "Image",
-    "drawing": "Image",
-    "photo": "Image",
-    "image": "Image",
-    "video": "Audiovisual",
+    "publication": "JournalArticle",
+    "publication-article": "JournalArticle",
+    "publication-blogpost": "BlogPost",
+    "publication-book": "Book",
+    "publication-conferencepaper": "ProceedingsArticle",
+    "publication-conferenceproceeding": "Proceedings",
+    "publication-datamanagementplan": "DataManagementPlan",
+    "publication-datapaper": "DataPaper",
+    "publication-dissertation": "Dissertation",
+    "publication-journal": "Journal",
+    "publication-other": "Other",
+    "publication-patent": "Patent",
+    "publication-peerreview": "PeerReview",
+    "publication-preprint": "Article",
+    "publication-report": "Report",
+    "publication-section": "BookChapter",
+    "publication-standard": "Standard",
+    "publication-technicalnote": "Report",
+    "publication-thesis": "Dissertation",
+    "publication-workingpaper": "Report",
+    "report": "Report",
+    "section": "BookChapter",
     "software": "Software",
-    "lesson": "InteractiveResource",
-    "physicalobject": "PhysicalObject",
+    "softwaredocumentation": "Software",
+    "taxonomictreatment": "Collection",
+    "technicalnote": "Report",
+    "thesis": "Dissertation",
+    "video": "Audiovisual",
     "workflow": "Workflow",
-    "other": "Other",
+    "workingpaper": "Report",
 }
 
 CM_TO_INVENIORDM_TRANSLATIONS = {
@@ -682,6 +699,7 @@ INVENIORDM_IDENTIFIER_TYPES = {
 
 CROSSREF_FUNDER_ID_TO_ROR_TRANSLATIONS = {
     "https://doi.org/10.13039/100000001": "https://ror.org/021nxhr62",
+    "https://doi.org/10.13039/100000015": "https://ror.org/01bj3aw27",
     "https://doi.org/10.13039/501100000780": "https://ror.org/00k4n6c32",
     "https://doi.org/10.13039/501100007601": "https://ror.org/00k4n6c32",
     "https://doi.org/10.13039/501100001659": "https://ror.org/018mejw64",
