@@ -5,9 +5,8 @@ from typing import Any, Dict, List, Optional, Union
 
 import orjson as json
 import yaml
-from pydash import py_
 
-from .base_utils import parse_xml, wrap
+from .base_utils import dig, omit, parse_xml, wrap
 from .file_utils import write_output
 from .readers.cff_reader import get_cff, read_cff
 from .readers.codemeta_reader import (
@@ -45,11 +44,12 @@ from .readers.schema_org_reader import (
     read_schema_org,
 )
 from .schema_utils import json_schema_errors, xml_schema_errors
-from .utils import find_from_format, normalize_id
+from .utils import find_from_format, is_chain_object, normalize_id
 from .writers.bibtex_writer import write_bibtex, write_bibtex_list
 from .writers.citation_writer import write_citation, write_citation_list
 from .writers.commonmeta_writer import write_commonmeta, write_commonmeta_list
 from .writers.crossref_xml_writer import (
+    push_crossref_xml,
     push_crossref_xml_list,
     write_crossref_xml,
     write_crossref_xml_list,
@@ -57,6 +57,7 @@ from .writers.crossref_xml_writer import (
 from .writers.csl_writer import write_csl, write_csl_list
 from .writers.datacite_writer import write_datacite, write_datacite_list
 from .writers.inveniordm_writer import (
+    push_inveniordm,
     push_inveniordm_list,
     write_inveniordm,
     write_inveniordm_list,
@@ -70,11 +71,20 @@ class Metadata:
     """Metadata"""
 
     def __init__(self, string: Optional[Union[str, Dict[str, Any]]], **kwargs):
-        if string is None or not isinstance(string, (str, dict)):
+        if (
+            string is None
+            or not isinstance(string, (str, dict))
+            and not is_chain_object
+        ):
             raise ValueError("No input found")
         self.via = kwargs.get("via", None)
         if isinstance(string, dict):
             data = string
+        # if string is an InvenioRDM chain object or any ChainObject-like object
+        elif is_chain_object:
+            data = string._child
+            self.via = "inveniordm"
+            kwargs["parent_doi"] = dig(string._parent.pids, "doi.identifier")
         elif isinstance(string, str):
             pid = normalize_id(string)
             if pid is not None and self.via is None:
@@ -108,6 +118,7 @@ class Metadata:
         self.provider = meta.get("provider")
         self.publisher = meta.get("publisher")
         self.references = meta.get("references")
+        self.citations = meta.get("citations")
         self.relations = meta.get("relations")
         self.subjects = meta.get("subjects")
         self.titles = meta.get("titles")
@@ -122,6 +133,19 @@ class Metadata:
         self.date_updated = meta.get("date_updated")
         self.state = meta.get("state")
 
+        # options needed for Crossref DOI registration
+        self.depositor = kwargs.get("depositor", None)
+        self.email = kwargs.get("email", None)
+        self.registrant = kwargs.get("registrant", None)
+        self.login_id = kwargs.get("login_id", None)
+        self.login_passwd = kwargs.get("login_passwd", None)
+        self.test_mode = kwargs.get("test_mode", False)
+
+        # options needed for InvenioRDM registration
+        self.host = kwargs.get("host", None)
+        self.token = kwargs.get("token", None)
+        self.legacy_key = kwargs.get("legacy_key", None)
+
         # Catch errors in the reader, then validate against JSON schema for Commonmeta
         self.errors = meta.get("errors", None) or json_schema_errors(
             json.loads(self.write())
@@ -241,7 +265,7 @@ class Metadata:
         elif via == "jsonfeed":
             return dict(read_jsonfeed(data, **kwargs))
         elif via == "inveniordm":
-            return dict(read_inveniordm(data))
+            return dict(read_inveniordm(data, **kwargs))
         elif via == "kbase":
             return dict(read_kbase(data))
         elif via == "openalex":
@@ -311,7 +335,7 @@ class Metadata:
         """Write in CSL format with error checking."""
         csl_output = write_csl(self)
         if csl_output:
-            instance = py_.omit(json.loads(csl_output), [])
+            instance = omit(json.loads(csl_output), [])
             self.errors = json_schema_errors(instance, schema="csl")
             return csl_output
         return ""
@@ -344,6 +368,29 @@ class Metadata:
             return ""
         return output if output is not None else ""
 
+    def push(self, to: str = "commonmeta", **kwargs) -> Union[str, bytes]:
+        """push metadata to external APIs"""
+
+        if to == "crossref_xml":
+            response = push_crossref_xml(
+                self,
+                login_id=self.login_id,
+                login_passwd=self.login_passwd,
+                test_mode=self.test_mode,
+                host=self.host,
+                token=self.token,
+                legacy_key=self.legacy_key,
+            )
+            return response
+        elif to == "datacite":
+            raise ValueError("Datacite not yet supported")
+        elif to == "inveniordm":
+            kwargs = {"legacy_key": self.legacy_key}
+            response = push_inveniordm(self, host=self.host, token=self.token, **kwargs)
+            return response
+        else:
+            raise ValueError("No valid output format found")
+
 
 class MetadataList:
     """MetadataList"""
@@ -355,13 +402,14 @@ class MetadataList:
             raise ValueError("No input found")
         if isinstance(dct, dict):
             meta = dct
+        elif isinstance(dct, list):
+            meta = {"items": dct}
         elif isinstance(dct, (str, bytes)):
             if path.exists(dct):
                 with open(dct, encoding="utf-8") as file:
                     dct = file.read()
             self.via = kwargs.get("via", None) or find_from_format(string=dct)
             meta = self.get_metadata_list(dct)
-
         self.id = meta.get("id", None)
         self.type = meta.get("type", None)
         self.title = meta.get("title", None)
@@ -373,6 +421,7 @@ class MetadataList:
         self.registrant = kwargs.get("registrant", None)
         self.login_id = kwargs.get("login_id", None)
         self.login_passwd = kwargs.get("login_passwd", None)
+        self.test_mode = kwargs.get("test_mode", False)
 
         # options needed for InvenioRDM registration
         self.host = kwargs.get("host", None)
@@ -392,12 +441,15 @@ class MetadataList:
     def get_metadata_list(self, string) -> list:
         if string is None or not isinstance(string, (str, bytes)):
             raise ValueError("No input found")
+        if self.via in [
+            "inveniordm",
+        ]:
+            return {"items": json.loads(string)}
         if self.via in [
             "commonmeta",
             "crossref",
             "csl",
             "datacite",
-            "inveniordm",
             "jsonfeed",
             "openalex",
             "schema_org",
@@ -470,6 +522,9 @@ class MetadataList:
                 self,
                 login_id=self.login_id,
                 login_passwd=self.login_passwd,
+                test_mode=self.test_mode,
+                host=self.host,
+                token=self.token,
                 legacy_key=self.legacy_key,
             )
             return response
