@@ -3,7 +3,7 @@
 import os
 import re
 import time
-from typing import Optional
+from typing import Any, Optional
 from urllib.parse import urlparse
 
 import bibtexparser
@@ -12,9 +12,10 @@ import pycountry
 import yaml
 from bs4 import BeautifulSoup
 from furl import furl
-from pydash import py_
+from idutils import is_isni, is_orcid
+from isbnlib import canonical, is_isbn10, is_isbn13
 
-from .base_utils import compact, parse_attributes, wrap
+from .base_utils import compact, dig, omit, parse_attributes, pascal_case, unique, wrap
 from .constants import DATACITE_CONTRIBUTOR_TYPES
 from .doi_utils import doi_as_url, doi_from_url, get_doi_ra, normalize_doi, validate_doi
 
@@ -319,6 +320,8 @@ def validate_orcid(orcid: Optional[str]) -> Optional[str]:
     if match is None:
         return None
     orcid = match.group(1).replace(" ", "-")
+    if not is_orcid(orcid):
+        return None
     return orcid
 
 
@@ -332,10 +335,22 @@ def validate_isni(isni: Optional[str]) -> Optional[str]:
     )
     if match is None:
         return None
-    isni = match.group(1).replace(" ", "")
+    isni = match.group(1).replace(" ", "").replace("-", "")
+    if not is_isni(isni):
+        return None
     return isni
 
 
+def validate_isbn(isbn: Optional[str]) -> Optional[str]:
+    """Validate ISBN"""
+    if isbn is None or not isinstance(isbn, str):
+        return None
+    isbn = canonical(isbn)
+    if not (is_isbn10(isbn) or is_isbn13(isbn)):
+        return None
+    return isbn
+
+
 def validate_mag(mag: Optional[str]) -> Optional[str]:
     """Validate Microsoft Academic Graph ID (mag)"""
     if mag is None or not isinstance(mag, str):
@@ -778,9 +793,7 @@ def from_crossref_xml(elements: list) -> list:
         if element.get("ORCID", None) is not None:
             orcid = parse_attributes(element.get("ORCID"))
             element["ORCID"] = normalize_orcid(orcid)
-        element = py_.omit(
-            element, "given_name", "surname", "sequence", "contributor_role"
-        )
+        element = omit(element, "given_name", "surname", "sequence", "contributor_role")
         return compact(element)
 
     return [format_element(i) for i in elements]
@@ -791,7 +804,7 @@ def from_kbase(elements: list) -> list:
 
     def map_contributor_role(role):
         if role.split(":")[0] == "CRediT":
-            return py_.pascal_case(role.split(":")[1])
+            return pascal_case(role.split(":")[1])
         elif role.split(":")[0] == "DataCite":
             return DATACITE_CONTRIBUTOR_TYPES.get(role.split(":")[1], "Other")
         else:
@@ -807,7 +820,7 @@ def from_kbase(elements: list) -> list:
             map_contributor_role(i)
             for i in wrap(element.get("contributor_roles", None))
         ]
-        element = py_.omit(element, "contributor_id")
+        element = omit(element, "contributor_id")
         return compact(element)
 
     return [format_element(i) for i in elements]
@@ -832,7 +845,7 @@ def from_csl(elements: list) -> list:
         element["givenName"] = element.get("given", None)
         element["familyName"] = element.get("family", None)
         element["affiliation"] = element.get("affiliation", None)
-        element = py_.omit(element, "given", "family", "literal", "sequence")
+        element = omit(element, "given", "family", "literal", "sequence")
         return compact(element)
 
     return [format_element(i) for i in elements]
@@ -898,7 +911,7 @@ def to_schema_org_creators(elements: list) -> list():
             element["@type"] = "Person"
         else:
             element["@type"] = "Organization"
-        element = py_.omit(element, "type", "contributorRoles")
+        element = omit(element, "type", "contributorRoles")
         return compact(element)
 
     return [format_element(i) for i in elements]
@@ -951,9 +964,9 @@ def to_schema_org_relations(related_items: list, relation_type=None):
     else:
         relation_type = [relation_type]
 
-    related_items = py_.filter(
-        wrap(related_items), lambda ri: ri["relationType"] in relation_type
-    )
+    related_items = [
+        ri for ri in wrap(related_items) if ri["relationType"] in relation_type
+    ]
     return [format_element(i) for i in related_items]
 
 
@@ -1037,11 +1050,11 @@ def find_from_format_by_dict(dct: dict) -> Optional[str]:
         return "datacite"
     if dct.get("source", None) == "Crossref":
         return "crossref"
-    if py_.get(dct, "issued.date-parts") is not None:
+    if dig(dct, "issued.date-parts") is not None:
         return "csl"
-    if py_.get(dct, "conceptdoi") is not None:
+    if dig(dct, "conceptdoi") is not None:
         return "inveniordm"
-    if py_.get(dct, "credit_metadata") is not None:
+    if dig(dct, "credit_metadata") is not None:
         return "kbase"
     return None
 
@@ -1072,11 +1085,11 @@ def find_from_format_by_string(string: str) -> Optional[str]:
             return "datacite"
         if data.get("source", None) == "Crossref":
             return "crossref"
-        if py_.get(data, "issued.date-parts") is not None:
+        if dig(data, "issued.date-parts") is not None:
             return "csl"
-        if py_.get(data, "conceptdoi") is not None:
+        if dig(data, "conceptdoi") is not None:
             return "inveniordm"
-        if py_.get(data, "credit_metadata") is not None:
+        if dig(data, "credit_metadata") is not None:
             return "kbase"
     except (TypeError, json.JSONDecodeError):
         pass
@@ -1127,7 +1140,7 @@ def from_schema_org(element):
         return None
     element["type"] = element.get("@type", None)
     element["id"] = element.get("@id", None)
-    return compact(py_.omit(element, ["@type", "@id"]))
+    return compact(omit(element, ["@type", "@id"]))
 
 
 def from_schema_org_creators(elements: list) -> list:
@@ -1144,8 +1157,9 @@ def from_schema_org_creators(elements: list) -> list:
         elif isinstance(i.get("@type", None), str):
             element["type"] = i.get("@type")
         elif isinstance(i.get("@type", None), list):
-            element["type"] = py_.find(
-                i["@type"], lambda x: x in ["Person", "Organization"]
+            # Find first element that matches the condition
+            element["type"] = next(
+                (x for x in i["@type"] if x in ["Person", "Organization"]), None
             )
 
         # strip text after comma if suffix is an academic title
@@ -1198,7 +1212,7 @@ def from_schema_org_creators(elements: list) -> list:
 
         if isinstance(i.get("affiliation", None), str):
             element["affiliation"] = {"type": "Organization", "name": i["affiliation"]}
-        elif urlparse(py_.get(i, "affiliation.@id", "")).hostname in [
+        elif urlparse(dig(i, "affiliation.@id", "")).hostname in [
             "ror.org",
             "isni.org",
         ]:
@@ -1367,6 +1381,15 @@ def name_to_fos(name: str) -> Optional[dict]:
     return {"subject": subject}
 
 
+def dict_to_fos(dct: dict) -> Optional[dict]:
+    """Convert dict to Fields of Science (OECD) subject"""
+    if not isinstance(dct, dict):
+        return None
+    if dct.get("subject", None) is not None:
+        return name_to_fos(dct["subject"])
+    return None
+
+
 def from_curie(id: Optional[str]) -> Optional[str]:
     """from CURIE"""
     if id is None:
@@ -1433,7 +1456,7 @@ def extract_urls(string: str) -> list:
         r"((?:http|https):\/\/(?:[\w_-]+(?:(?:\.[\w_-]+)+))(?:[\w.,@?^=%&:\/~+#-]*[\w@?^=%&\/~+#-]))",
         string,
     )
-    return py_.uniq(urls)
+    return unique(urls)
 
 
 def issn_as_url(issn: str) -> Optional[str]:
@@ -1509,3 +1532,15 @@ def id_from_url(url: Optional[str]) -> Optional[str]:
         return None
 
     return str(f.path).strip("/")
+
+
+def is_chain_object(string: Any) -> bool:
+    """Check if the given string is a ChainObject-like object,
+    for example the invenio-rdm-records ChainObject."""
+    if string is None:
+        return False
+    return (
+        hasattr(string, "_objs")
+        and hasattr(string, "__getattr__")
+        and hasattr(string, "__getitem__")
+    )
