@@ -4,23 +4,23 @@ import io
 import logging
 from datetime import datetime
 from time import time
-from typing import Dict, Optional
+from typing import Dict, Optional, Union
 
 import orjson as json
 import requests
 from dateutil.parser import parse as date_parse
 from furl import furl
+from isbnlib import canonical, is_isbn10, is_isbn13
 from marshmallow import Schema, fields
-from pydash import py_
 from requests_toolbelt.multipart.encoder import MultipartEncoder
 
-from ..base_utils import compact, parse_xml, unparse_xml, unparse_xml_list, wrap
+from ..base_utils import compact, dig, parse_xml, unparse_xml, unparse_xml_list, wrap
 from ..constants import Commonmeta
 from ..doi_utils import doi_from_url, is_rogue_scholar_doi, validate_doi
 from ..utils import validate_url
-from .inveniordm_writer import update_legacy_record
+from .inveniordm_writer import push_inveniordm, update_legacy_record
 
-logger = logging.getLogger(__name__)
+log = logging.getLogger(__name__)
 
 POSTED_CONTENT_TYPES = [
     "preprint",
@@ -108,10 +108,12 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
         "Report",
         "Standard",
     ]:
+        log.error(f"Type not supported by Crossref: {metadata.id}")
         return None
 
     # return None if doi or url are not present
     if doi_from_url(metadata.id) is None or metadata.url is None:
+        log.error(f"DOI or URL missing for Crossref XML: {metadata.id}")
         return None
 
     titles = get_titles(metadata)
@@ -177,7 +179,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
                 "titles": titles,
                 "abstracts": abstracts,
                 "publication_date": get_publication_date(metadata, media_type="online"),
-                "isbn": get_isbn(metadata),
+                "isbn": get_isbn(metadata, media_type="online"),
                 "publisher": get_publisher(metadata),
                 "publisher_item": None,
                 "funding_references": funding_references,
@@ -218,7 +220,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
             }
         )
     elif metadata.type == "Dataset":
-        publisher = py_.get(metadata, "publisher.name")
+        publisher = dig(metadata, "publisher.name")
         if publisher is not None:
             publisher_item = {
                 "title": publisher,
@@ -292,14 +294,16 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
             }
         )
     elif metadata.type == "ProceedingsArticle":
-        publisher_item = None
+        publisher = dig(metadata, "publisher.name")
+        if publisher is not None:
+            publisher_item = {
+                "title": publisher,
+            }
         data = compact(
             {
                 "conference": get_attributes(metadata, **kwargs),
                 "event_metadata": get_event_metadata(metadata),
                 "proceedings_metadata": get_proceedings_metadata(metadata),
-                "proceedings_title": py_.get(metadata, "container.title"),
-                "publisher": get_publisher(metadata),
                 "conference_paper": get_attributes(metadata, **kwargs),
                 "contributors": contributors,
                 "titles": titles,
@@ -337,42 +341,54 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
             }
         )
     else:
+        log.error(f"Another error occured for Crossref XML: {metadata.id}")
         data = None
     return data
 
 
 def write_crossref_xml(metadata: Commonmeta) -> Optional[str]:
     """Write Crossref XML"""
-
-    data = convert_crossref_xml(metadata)
-    if data is None:
-        logger.error(f"Could not convert metadata to Crossref XML: {metadata.id}")
+    if metadata is None or not metadata.is_valid:
+        log.error("Invalid metadata provided for Crossref XML generation")
         return None
 
     # Use the marshmallow schema to dump the data
     schema = CrossrefXMLSchema()
+
+    data = convert_crossref_xml(metadata)
+    if data is None:
+        log.error(f"Could not convert metadata to Crossref XML: {metadata.id}")
+        return None
+
     crossref_xml = schema.dump(data)
 
     # Ensure consistent field ordering through the defined mapping
     field_order = [MARSHMALLOW_MAP.get(k, k) for k in list(data.keys())]
     crossref_xml = {k: crossref_xml[k] for k in field_order if k in crossref_xml}
 
+    head = {
+        "depositor": metadata.depositor,
+        "email": metadata.email,
+        "registrant": metadata.registrant,
+    }
+
     # Convert to XML
-    return unparse_xml(crossref_xml, dialect="crossref")
+    return unparse_xml(crossref_xml, dialect="crossref", head=head)
 
 
 def write_crossref_xml_list(metalist) -> Optional[str]:
     """Write crossref_xml list"""
     if metalist is None or not metalist.is_valid:
-        logger.error("Invalid metalist provided for Crossref XML generation")
+        log.error("Invalid metalist provided for Crossref XML generation")
         return None
 
+    # Use the marshmallow schema to dump the data
     schema = CrossrefXMLSchema()
     crossref_xml_list = []
     for item in metalist.items:
         data = convert_crossref_xml(item)
         if data is None or not isinstance(data, dict):
-            logger.error(f"Could not convert metadata to Crossref XML: {item.id}")
+            log.error(f"Could not convert metadata to Crossref XML: {item.id}")
             continue
 
         crossref_xml = schema.dump(data)
@@ -389,53 +405,88 @@ def write_crossref_xml_list(metalist) -> Optional[str]:
     return unparse_xml_list(crossref_xml_list, dialect="crossref", head=head)
 
 
+def push_crossref_xml(
+    metadata: Commonmeta,
+    login_id: str,
+    login_passwd: str,
+    test_mode: bool,
+    host: str,
+    token: str,
+    legacy_key: str,
+) -> str:
+    """Push crossref_xml to Crossref API, returns the API response."""
+
+    input_xml = write_crossref_xml(metadata)
+    if not input_xml:
+        log.error("Failed to generate XML for upload")
+        return "{}"
+
+    client = CrossrefXMLClient(
+        username=login_id,
+        password=login_passwd,
+        test_mode=test_mode,
+    )
+    status = client.post(input_xml)
+
+    if status != "SUCCESS":
+        log.error("Failed to upload XML to Crossref")
+        return "{}"
+
+    record = {
+        "doi": metadata.id,
+        "updated": datetime.now().isoformat("T", "seconds"),
+        "status": "submitted",
+    }
+
+    # update rogue-scholar record with state findable
+    # if state is stale and host and token are provided
+    if (
+        is_rogue_scholar_doi(metadata.id, ra="crossref")
+        and metadata.state == "stale"
+        and host is not None
+        and token is not None
+    ):
+        metadata.state = "findable"
+        r = push_inveniordm(metadata, host=host, token=token)
+        if r["status"] != "error":
+            record["status"] = "updated"
+
+    # update rogue-scholar legacy record if legacy_key is provided
+    if is_rogue_scholar_doi(metadata.id, ra="crossref") and legacy_key is not None:
+        uuid = dig(metadata, "identifiers.0.identifier")
+        if uuid:
+            record["uuid"] = uuid
+            record = update_legacy_record(record, legacy_key=legacy_key, field="doi")
+
+    # Return JSON response
+    return json.dumps(record, option=json.OPT_INDENT_2).decode("utf-8")
+
+
 def push_crossref_xml_list(
-    metalist, login_id: str, login_passwd: str, legacy_key: Optional[str] = None
+    metalist,
+    login_id: str,
+    login_passwd: str,
+    test_mode: bool,
+    host: str,
+    token: str,
+    legacy_key: str,
 ) -> str:
     """Push crossref_xml list to Crossref API, returns the API response."""
 
     input_xml = write_crossref_xml_list(metalist)
     if not input_xml:
-        logger.error("Failed to generate XML for upload")
+        log.error("Failed to generate XML for upload")
         return "{}"
 
-    # Convert string to bytes if necessary
-    if isinstance(input_xml, str):
-        input_xml = input_xml.encode("utf-8")
-
-    # The filename displayed in the Crossref admin interface
-    filename = f"{int(time())}"
-
-    multipart_data = MultipartEncoder(
-        fields={
-            "fname": (filename, io.BytesIO(input_xml), "application/xml"),
-            "operation": "doMDUpload",
-            "login_id": login_id,
-            "login_passwd": login_passwd,
-        }
+    client = CrossrefXMLClient(
+        username=login_id,
+        password=login_passwd,
+        test_mode=test_mode,
     )
+    status = client.post(input_xml)
 
-    post_url = "https://doi.crossref.org/servlet/deposit"
-    headers = {"Content-Type": multipart_data.content_type}
-
-    try:
-        resp = requests.post(post_url, data=multipart_data, headers=headers, timeout=30)
-        resp.raise_for_status()
-    except requests.exceptions.RequestException as e:
-        logger.error(f"Failed to upload to Crossref: {e}")
-        return "{}"
-
-    # Parse the response
-    try:
-        response = parse_xml(resp.content)
-        status = py_.get(response, "html.body.h2")
-        if status != "SUCCESS":
-            # Handle error response
-            message = py_.get(response, "html.body.p")
-            logger.error(f"Crossref API error: {message}")
-            return "{}"
-    except Exception as e:
-        logger.error(f"Failed to parse Crossref response: {e}")
+    if status != "SUCCESS":
+        log.error("Failed to upload XML to Crossref")
         return "{}"
 
     items = []
@@ -446,9 +497,22 @@ def push_crossref_xml_list(
             "status": "submitted",
         }
 
+        # update rogue-scholar record with state findable
+        # if state is stale and host and token are provided
+        if (
+            is_rogue_scholar_doi(item.id, ra="crossref")
+            and item.state == "stale"
+            and host is not None
+            and token is not None
+        ):
+            item.state = "findable"
+            r = push_inveniordm(item, host=host, token=token)
+            if r["status"] != "error":
+                record["status"] = "updated"
+
         # update rogue-scholar legacy record if legacy_key is provided
         if is_rogue_scholar_doi(item.id, ra="crossref") and legacy_key is not None:
-            uuid = py_.get(item, "identifiers.0.identifier")
+            uuid = dig(item, "identifiers.0.identifier")
             if uuid:
                 record["uuid"] = uuid
                 record = update_legacy_record(
@@ -476,14 +540,14 @@ def get_attributes(obj, **kwargs) -> dict:
 def get_journal_metadata(obj) -> Optional[dict]:
     """get journal metadata"""
     issn = (
-        py_.get(obj, "container.identifier")
-        if py_.get(obj, "container.identifierType") == "ISSN"
+        dig(obj, "container.identifier")
+        if dig(obj, "container.identifierType") == "ISSN"
         else None
     )
     return compact(
         {
-            "@language": py_.get(obj, "language"),
-            "full_title": py_.get(obj, "container.title"),
+            "@language": dig(obj, "language"),
+            "full_title": dig(obj, "container.title"),
             "issn": issn,
         }
     )
@@ -492,7 +556,7 @@ def get_journal_metadata(obj) -> Optional[dict]:
 def get_book_metadata(obj) -> Optional[dict]:
     return compact(
         {
-            "@language": py_.get(obj, "language"),
+            "@language": dig(obj, "language"),
         }
     )
 
@@ -500,20 +564,20 @@ def get_book_metadata(obj) -> Optional[dict]:
 def get_database_metadata(obj) -> Optional[dict]:
     return compact(
         {
-            "@language": py_.get(obj, "language"),
+            "@language": dig(obj, "language"),
         }
     )
 
 
 def get_event_metadata(obj) -> Optional[dict]:
     """get event metadata"""
-    if py_.get(obj, "container.title") is None:
+    if dig(obj, "container.title") is None:
         return None
 
     return compact(
         {
-            "conference_name": py_.get(obj, "container.title"),
-            "conference_location": py_.get(obj, "container.location"),
+            "conference_name": dig(obj, "container.title"),
+            "conference_location": dig(obj, "container.location"),
             "conference_date": None,
         }
     )
@@ -521,54 +585,51 @@ def get_event_metadata(obj) -> Optional[dict]:
 
 def get_proceedings_metadata(obj) -> Optional[dict]:
     """get proceedings metadata"""
-    if py_.get(obj, "container.title") is None:
+    if dig(obj, "container.title") is None or dig(obj, "publisher.name") is None:
         return None
 
-    return compact(
-        {
-            "@language": py_.get(obj, "language"),
-            "proceedings_title": py_.get(obj, "container.title"),
-        }
-    )
+    proceedings_metadata = {
+        "@language": dig(obj, "language"),
+        "proceedings_title": dig(obj, "container.title"),
+        "publisher": {
+            "publisher_name": dig(obj, "publisher.name"),
+        },
+        "publication_date": get_publication_date(obj, media_type="online"),
+    }
+    if dig(obj, "container.identifierType") == "ISBN":
+        proceedings_metadata["isbn"] = get_isbn(obj)
+    return compact(proceedings_metadata)
 
 
 def get_journal_issue(obj) -> Optional[dict]:
     """get journal issue"""
-    volume = py_.get(obj, "container.volume")
+    volume = dig(obj, "container.volume")
     if volume is not None:
         volume = {"volume": volume}
     return compact(
         {
             "publication_date": get_publication_date(obj),
             "journal_volume": volume,
-            "issue": py_.get(obj, "container.issue"),
+            "issue": dig(obj, "container.issue"),
         }
     )
 
 
 def get_institution(obj) -> Optional[dict]:
     """get institution"""
-    if py_.get(obj, "container.title") is None:
+    if dig(obj, "publisher.name") is None:
         return None
 
-    return compact(
-        {
-            "institution_name": py_.get(obj, "container.title"),
-            "institution_id": {
-                "#text": py_.get(obj, "container.identifier"),
-                "@type": "ror",
-            }
-            if py_.get(obj, "container.identifierTyoe") == "ROR"
-            else None,
-        }
-    )
+    return {
+        "institution_name": dig(obj, "publisher.name"),
+    }
 
 
 def get_titles(obj) -> Optional[dict]:
     """get titles"""
 
     titles = {}
-    for t in wrap(py_.get(obj, "titles", [])):
+    for t in wrap(dig(obj, "titles", [])):
         if isinstance(t, str):
             titles["title"] = t
         elif isinstance(t, dict) and t.get("titleType", None) == "Subtitle":
@@ -605,12 +666,12 @@ def get_contributors(obj) -> Optional[dict]:
             for affiliation in affiliations
         ]
 
-    if py_.get(obj, "contributors") is None or len(py_.get(obj, "contributors")) == 0:
+    if len(wrap(dig(obj, "contributors"))) == 0:
         return None
 
     con = [
         c
-        for c in py_.get(obj, "contributors")
+        for c in dig(obj, "contributors", [])
         if c.get("contributorRoles", None) == ["Author"]
         or c.get("contributorRoles", None) == ["Editor"]
     ]
@@ -683,21 +744,21 @@ def get_contributors(obj) -> Optional[dict]:
 
 def get_publisher(obj) -> Optional[dict]:
     """get publisher"""
-    if py_.get(obj, "publisher.name") is None:
+    if dig(obj, "publisher.name") is None:
         return None
 
     return {
-        "publisher_name": py_.get(obj, "publisher.name"),
+        "publisher_name": dig(obj, "publisher.name"),
     }
 
 
 def get_abstracts(obj) -> Optional[list]:
     """get abstracts"""
-    if py_.get(obj, "descriptions") is None:
+    if len(wrap(dig(obj, "descriptions"))) == 0:
         return None
 
     abstracts = []
-    for d in wrap(py_.get(obj, "descriptions", [])):
+    for d in wrap(dig(obj, "descriptions", [])):
         if d.get("type", None) == "Abstract":
             abstracts.append(
                 {
@@ -717,23 +778,15 @@ def get_abstracts(obj) -> Optional[list]:
 
 def get_group_title(obj) -> Optional[str]:
     """Get group title from metadata"""
-    if py_.get(obj, "subjects") is None or len(py_.get(obj, "subjects")) == 0:
-        return None
-    group_title = py_.get(obj, "subjects[0].subject")
-
-    # strip optional FOS (Field of Science) prefix
-    if group_title.startswith("FOS: "):
-        group_title = group_title[5:]
-
-    return group_title
+    return dig(obj, "container.title")
 
 
 def get_item_number(obj) -> Optional[dict]:
     """Insert item number"""
-    if py_.get(obj, "identifiers") is None:
+    if len(wrap(dig(obj, "identifiers"))) == 0:
         return None
 
-    for identifier in py_.get(obj, "identifiers"):
+    for identifier in wrap(dig(obj, "identifiers")):
         if identifier.get("identifierType", None) == "UUID":
             # strip hyphen from UUIDs, as item_number can only be 32 characters long (UUIDv4 is 36 characters long)
             return {
@@ -744,14 +797,14 @@ def get_item_number(obj) -> Optional[dict]:
 
 def get_publication_date(obj, media_type: Optional[str] = None) -> Optional[Dict]:
     """get publication date"""
-    pub_date_str = py_.get(obj, "date.published")
+    pub_date_str = dig(obj, "date.published")
     if pub_date_str is None:
         return None
 
     try:
         pub_date = date_parse(pub_date_str)
     except (ValueError, TypeError) as e:
-        logger.warning(f"Failed to parse publication date '{pub_date_str}': {e}")
+        log.warning(f"Failed to parse publication date '{pub_date_str}': {e}")
         return None
 
     return compact(
@@ -766,10 +819,7 @@ def get_publication_date(obj, media_type: Optional[str] = None) -> Optional[Dict
 
 def get_archive_locations(obj) -> Optional[list]:
     """get archive locations"""
-    if (
-        py_.get(obj, "archive_locations") is None
-        or len(py_.get(obj, "archive_locations")) == 0
-    ):
+    if len(wrap(dig(obj, "archive_locations"))) == 0:
         return None
 
     return [
@@ -778,19 +828,29 @@ def get_archive_locations(obj) -> Optional[list]:
                 "archive": {"@name": location},
             }
         )
-        for location in py_.get(obj, "archive_locations")
+        for location in dig(obj, "archive_locations")
     ]
 
 
 def get_references(obj) -> Optional[Dict]:
     """get references"""
-    if py_.get(obj, "references") is None or len(py_.get(obj, "references")) == 0:
+    if len(wrap(dig(obj, "references"))) == 0:
         return None
 
     citations = []
-    for i, ref in enumerate(py_.get(obj, "references")):
+    for i, ref in enumerate(dig(obj, "references", [])):
         # Validate DOI before using it
         doi = doi_from_url(ref.get("id", None))
+        unstructured = ref.get("unstructured", None)
+
+        # include id in unstructured_citation if it is not a DOI
+        if (
+            doi is None
+            and unstructured is not None
+            and ref.get("id", None) is not None
+            and not unstructured.endswith(ref.get("id"))
+        ):
+            unstructured += " " + ref.get("id")
 
         reference = compact(
             {
@@ -802,7 +862,7 @@ def get_references(obj) -> Optional[Dict]:
                 "first_page": ref.get("first_page", None),
                 "cYear": ref.get("publicationYear", None),
                 "article_title": ref.get("title", None),
-                "unstructured_citation": ref.get("unstructured", None),
+                "unstructured_citation": unstructured,
             }
         )
         citations.append(reference)
@@ -811,7 +871,7 @@ def get_references(obj) -> Optional[Dict]:
 
 def get_license(obj) -> Optional[dict]:
     """get license"""
-    rights_uri = py_.get(obj, "license.url")
+    rights_uri = dig(obj, "license.url")
     if rights_uri is None:
         return None
 
@@ -833,47 +893,92 @@ def get_license(obj) -> Optional[dict]:
 
 def get_funding_references(obj) -> Optional[dict]:
     """Get funding references"""
-    if (
-        py_.get(obj, "funding_references") is None
-        or len(py_.get(obj, "funding_references")) == 0
-    ):
+    if len(wrap(dig(obj, "funding_references"))) == 0:
         return None
 
-    funding_references = []
-    for funding_reference in wrap(py_.get(obj, "funding_references")):
+    funding_refs = wrap(dig(obj, "funding_references"))
+    assertions = []
+
+    # Check if we need funding groups (multiple funders with award numbers)
+    funders_with_awards = [
+        f for f in funding_refs if f.get("awardNumber", None) is not None
+    ]
+    unique_funders = set(
+        f.get("funderIdentifier", None) or f.get("funderName", None)
+        for f in funding_refs
+        if f.get("funderIdentifier", None) is not None
+        or f.get("funderName", None) is not None
+    )
+
+    use_funding_groups = len(funders_with_awards) > 0 and len(unique_funders) > 1
+
+    for funding_reference in funding_refs:
+        group_assertions = []
+
+        # Handle funder identifier/name
         funder_identifier = funding_reference.get("funderIdentifier", None)
         funder_identifier_type = funding_reference.get("funderIdentifierType", None)
+        funder_name = funding_reference.get("funderName", None)
 
-        if funder_identifier is not None and funder_identifier_type == "ROR":
-            assertion = {
+        if funder_identifier and funder_identifier_type == "ROR":
+            funder_assertion = {
                 "@name": "ror",
                 "#text": funder_identifier,
             }
-
-            funding_references.append(assertion)
-        elif funding_reference.get("funderName", None) is not None:
-            assertion = {
+            group_assertions.append(funder_assertion)
+        elif funder_identifier and funder_identifier_type == "Crossref Funder ID":
+            # Create nested structure for Crossref Funder ID
+            funder_id_assertion = {
+                "@name": "funder_identifier",
+                "#text": funder_identifier,
+            }
+            funder_assertion = {
                 "@name": "funder_name",
-                "#text": funding_reference.get("funderName"),
+                "#text": funder_name,
+                "fr:assertion": [funder_id_assertion],
             }
-            funding_references.append(assertion)
+            group_assertions.append(funder_assertion)
+        elif funder_name:
+            funder_assertion = {
+                "@name": "funder_name",
+                "#text": funder_name,
+            }
+            group_assertions.append(funder_assertion)
 
-        if funding_reference.get("awardNumber", None) is not None:
-            assertion = {
+        # Handle award number
+        award_number = funding_reference.get("awardNumber", None)
+        if award_number:
+            award_assertion = {
                 "@name": "award_number",
-                "#text": funding_reference.get("awardNumber"),
+                "#text": award_number,
             }
-            funding_references.append(assertion)
+            group_assertions.append(award_assertion)
+
+        # Add to main assertions list
+        if use_funding_groups:
+            # Wrap in funding group
+            funding_group = {
+                "@name": "fundgroup",
+                "fr:assertion": group_assertions,
+            }
+            assertions.append(funding_group)
+        else:
+            # Add assertions directly
+            assertions.extend(group_assertions)
+
+    if not assertions:
+        return None
+
     return {
         "@xmlns:fr": "http://www.crossref.org/fundref.xsd",
         "@name": "fundref",
-        "fr:assertion": funding_references,
+        "fr:assertion": assertions,
     }
 
 
 def get_relations(obj) -> Optional[Dict]:
     """get relations"""
-    if py_.get(obj, "relations") is None or len(py_.get(obj, "relations")) == 0:
+    if len(wrap(dig(obj, "relations"))) == 0:
         return None
 
     def format_relation(relation):
@@ -925,7 +1030,7 @@ def get_relations(obj) -> Optional[Dict]:
         return {
             group: compact(
                 {
-                    "@relationship-type": py_.lower_first(relation_type),
+                    "@relationship-type": relation_type[0].lower() + relation_type[1:],
                     "@identifier-type": identifier_type,
                     "#text": _id,
                 },
@@ -934,7 +1039,7 @@ def get_relations(obj) -> Optional[Dict]:
 
     related_items = [
         format_relation(i)
-        for i in py_.get(obj, "relations")
+        for i in dig(obj, "relations")
         if format_relation(i) is not None
     ]
 
@@ -950,10 +1055,10 @@ def get_relations(obj) -> Optional[Dict]:
 
 def get_subjects(obj) -> Optional[list]:
     """Get crossref subjects"""
-    if py_.get(obj, "subjects") is None:
+    if dig(obj, "subjects") is None:
         return None
     subjects = []
-    for subject in py_.get(obj, "subjects"):
+    for subject in dig(obj, "subjects"):
         if isinstance(subject, dict):
             subjects.append(subject.get("subject", None))
         else:
@@ -963,18 +1068,18 @@ def get_subjects(obj) -> Optional[list]:
 
 def get_doi_data(obj) -> Optional[dict]:
     """get doi data"""
-    if doi_from_url(py_.get(obj, "id")) is None or py_.get(obj, "url") is None:
+    if doi_from_url(dig(obj, "id")) is None or dig(obj, "url") is None:
         return None
 
     items = [
         {
             "resource": {
                 "@mime_type": "text/html",
-                "#text": py_.get(obj, "url"),
+                "#text": dig(obj, "url"),
             }
         }
     ]
-    for file in wrap(py_.get(obj, "files")):
+    for file in wrap(dig(obj, "files")):
         if file.get("mimeType", None) is not None and file.get("url", None) is not None:
             items.append(
                 {
@@ -987,8 +1092,8 @@ def get_doi_data(obj) -> Optional[dict]:
 
     return compact(
         {
-            "doi": doi_from_url(py_.get(obj, "id")),
-            "resource": py_.get(obj, "url"),
+            "doi": doi_from_url(dig(obj, "id")),
+            "resource": dig(obj, "url"),
             "collection": {
                 "@property": "text-mining",
                 "item": items,
@@ -997,18 +1102,147 @@ def get_doi_data(obj) -> Optional[dict]:
     )
 
 
-def get_isbn(obj):
+def get_isbn(obj, media_type: Optional[str] = None) -> Optional[Dict]:
     """get isbn"""
-    if py_.get(obj, "container.identifierType") != "ISBN":
+    if (
+        dig(obj, "container.identifierType") != "ISBN"
+        or dig(obj, "container.identifier") is None
+    ):
         return None
-    return py_.get(obj, "container.identifier")
+    isbn = dig(obj, "container.identifier")
+    return normalize_isbn_crossref(isbn)
+
+
+def normalize_isbn_crossref(isbn: str) -> Optional[str]:
+    """Normalize ISBN for Crossref XML.
+
+    Crossref XSD pattern: (97(8|9)-)?\\d[\\d \\-]+[\\dX]
+    """
+    if not isbn:
+        return None
+
+    # Get canonical ISBN (removes hyphens and validates)
+    isbn_canonical = canonical(isbn)
+    if isbn_canonical is None:
+        return None
+
+    # For ISBN-13
+    if is_isbn13(isbn_canonical):
+        return (
+            f"{isbn_canonical[:3]}-{isbn_canonical[3:]}"  # Add hyphen after 978 or 979
+        )
+
+    # For ISBN-10
+    elif is_isbn10(isbn_canonical):
+        return isbn_canonical  # ISBN-10 is already in the correct format
+
+    # If we can't properly format it, return None rather than invalid format
+    return None
 
 
 def get_issn(obj):
     """get issn"""
-    if py_.get(obj, "container.identifierType") != "ISSN":
+    if dig(obj, "container.identifierType") != "ISSN":
         return None
-    return py_.get(obj, "container.identifier")
+    return dig(obj, "container.identifier")
+
+
+class CrossrefXMLClient:
+    """Crossref XML API client wrapper."""
+
+    def __init__(
+        self,
+        username: str,
+        password: str,
+        prefixes: list = ["10.5555"],
+        test_mode: bool = False,
+        timeout: int = 30,
+    ):
+        """Initialize the Crossref API client wrapper.
+
+        :param username: Crossref login ID.
+        :param password: Crossref login password.
+        :param timeout: Request timeout in seconds.
+        """
+        self.login_id = username
+        self.login_passwd = password
+        self.timeout = timeout
+        self.prefixes = [str(prefix) for prefix in prefixes]
+
+        if test_mode:
+            self.api_url = "https://test.crossref.org/servlet/deposit"
+        else:
+            self.api_url = "https://doi.crossref.org/servlet/deposit"
+
+    def post(self, input_xml: Union[str, bytes]) -> str:
+        """Upload metadata for a new or existing DOI.
+
+        :param input_xml: XML metadata following the Crossref Metadata Schema (str or bytes).
+        :return: Status string ('SUCCESS' or '{}' on error).
+        """
+        try:
+            # Convert string to bytes if necessary
+            if isinstance(input_xml, str):
+                input_xml = input_xml.encode("utf-8")
+
+            # The filename displayed in the Crossref admin interface
+            filename = f"{int(time())}"
+
+            multipart_data = MultipartEncoder(
+                fields={
+                    "fname": (filename, io.BytesIO(input_xml), "application/xml"),
+                    "operation": "doMDUpload",
+                    "login_id": self.login_id,
+                    "login_passwd": self.login_passwd,
+                }
+            )
+
+            headers = {"Content-Type": multipart_data.content_type}
+
+            # Make the request
+            resp = requests.post(
+                self.api_url, data=multipart_data, headers=headers, timeout=self.timeout
+            )
+            resp.raise_for_status()
+
+            return self._parse_response(resp)
+
+        except requests.exceptions.RequestException as e:
+            log.error(f"Failed to upload to Crossref: {e}")
+            return "{}"
+
+    def _parse_response(self, resp) -> str:
+        """Parse the Crossref API response.
+
+        :param resp: Response object from requests.
+        :return: Status string ('SUCCESS' or 'error message' on error).
+        """
+        try:
+            # Check content type
+            content_type = resp.headers.get("content-type", "").lower()
+            if "xml" not in content_type and "html" not in content_type:
+                log.error(f"Unexpected content type: {content_type}")
+                log.error(f"Response body: {resp.text}")
+                return "{}"
+
+            response = parse_xml(resp.content)
+
+            status = dig(response, "html.body.h2")
+            if status != "SUCCESS":
+                # Handle error response
+                message = dig(response, "html.body.p")
+                log.error(f"Crossref API error: {message}")
+                return message
+
+            return status
+
+        except Exception as e:
+            log.error(f"Failed to parse Crossref response: {e}")
+            log.error(
+                f"Response content type: {resp.headers.get('content-type', 'unknown')}"
+            )
+            log.error(f"Response body: {resp.text}")
+            return "{}"
 
 
 """Errors for the Crossref XML API.
@@ -1047,6 +1281,8 @@ class CrossrefError(Exception):
             return CrossrefForbiddenError(*args)
         elif err_code == 404:
             return CrossrefNotFoundError(*args)
+        elif err_code == 410:
+            return CrossrefGoneError(*args)
         else:
             return CrossrefServerError(*args)
 
@@ -1089,3 +1325,7 @@ class CrossrefForbiddenError(CrossrefRequestError):
 
 class CrossrefNotFoundError(CrossrefRequestError):
     """DOI does not exist in the database."""
+
+
+class CrossrefGoneError(CrossrefRequestError):
+    """DOI is no longer available."""
