@@ -5,10 +5,9 @@ from time import time
 from typing import Dict, Optional
 
 import orjson as json
-import pydash as py_
 import requests
 
-from ..base_utils import compact, parse_attributes, presence, wrap
+from ..base_utils import compact, dig, parse_attributes, presence, wrap
 from ..constants import (
     CM_TO_INVENIORDM_TRANSLATIONS,
     COMMUNITY_TRANSLATIONS,
@@ -17,7 +16,7 @@ from ..constants import (
     Commonmeta,
 )
 from ..date_utils import get_iso8601_date
-from ..doi_utils import doi_from_url, normalize_doi
+from ..doi_utils import doi_from_url, is_rogue_scholar_doi, normalize_doi
 from ..utils import (
     FOS_MAPPINGS,
     get_language,
@@ -29,7 +28,7 @@ from ..utils import (
     validate_ror,
 )
 
-logger = logging.getLogger(__name__)
+log = logging.getLogger(__name__)
 
 
 def write_inveniordm(metadata):
@@ -59,6 +58,7 @@ def write_inveniordm(metadata):
         }
     )
     references = [to_inveniordm_reference(i) for i in wrap(metadata.references)]
+    citations = [to_inveniordm_reference(i) for i in wrap(metadata.citations)]
     related_identifiers = [
         to_inveniordm_related_identifier(i)
         for i in wrap(metadata.relations)
@@ -109,7 +109,9 @@ def write_inveniordm(metadata):
             "pids": {
                 "doi": {
                     "identifier": doi_from_url(metadata.id),
-                    "provider": "external",
+                    "provider": "crossref"
+                    if is_rogue_scholar_doi(metadata.id, ra="crossref")
+                    else "external",
                 },
             },
             "access": {"record": "public", "files": "public"},
@@ -160,6 +162,7 @@ def write_inveniordm(metadata):
                     ),
                     "rs:content_html": presence(metadata.content),
                     "rs:image": presence(metadata.image),
+                    "rs:citations": presence(citations),
                     "rs:generator": container.get("platform", None),
                 }
             ),
@@ -382,9 +385,16 @@ def to_inveniordm_funding(funding: dict) -> Optional[dict]:
 
 def write_inveniordm_list(metalist):
     """Write InvenioRDM list"""
+
     if metalist is None:
         return None
-    return [write_inveniordm(item) for item in metalist.items]
+
+    def write_item(item):
+        """write inveniordm item for inveniordm list"""
+
+        return write_inveniordm(item)
+
+    return [write_item(item) for item in metalist.items]
 
 
 def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Dict:
@@ -405,11 +415,14 @@ def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Di
         # in the case of Rogue Scholar the blog community
 
         if hasattr(metadata, "identifiers") and metadata.identifiers:
-            for i, identifier in enumerate(metadata.identifiers):
-                if identifier.get("identifierType") == "UUID" and identifier.get(
-                    "identifier"
+            for identifier in metadata.identifiers:
+                if (
+                    isinstance(identifier, dict)
+                    and identifier.get("identifierType") == "UUID"
+                    and identifier.get("identifier")
                 ):
                     record["uuid"] = identifier.get("identifier")
+                    continue
 
         if hasattr(metadata, "relations") and metadata.relations:
             community_index = None
@@ -423,7 +436,7 @@ def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Di
                         record["community"] = slug
                         record["community_id"] = community_id
                         community_index = i
-                        break
+                        continue
 
             if community_index is not None:
                 metadata.relations.pop(community_index)
@@ -438,7 +451,8 @@ def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Di
         record = update_external_services(metadata, host, token, record, **kwargs)
 
     except Exception as e:
-        logger.error(
+        print(f"Error in push_inveniordm: {str(e)}")
+        log.error(
             f"Unexpected error in push_inveniordm: {str(e)}",
             exc_info=True,
             extra={"host": host, "record_id": record.get("id")},
@@ -465,14 +479,15 @@ def upsert_record(metadata: Commonmeta, host: str, token: str, record: dict) ->
     # Check if record already exists in InvenioRDM
     record["id"] = search_by_doi(doi_from_url(record.get("doi")), host, token)
 
-    if record["id"] is not None:
+    if record.get("id", None) is not None:
         # Create draft record from published record
         record = edit_published_record(record, host, token)
 
         # Update draft record
         record = update_draft_record(record, host, token, output)
     else:
-        # Create draft record
+        # Create draft record for DOI that is external or needs to be registered
+        # (currently only supported for Crossref PID provider)
         record = create_draft_record(record, host, token, output)
 
     # Publish draft record
@@ -512,7 +527,7 @@ def add_record_to_communities(
     # Add record to communities defined as IsPartOf relation in InvenioRDM RelatedIdentifiers
     if hasattr(metadata, "related_identifiers") and metadata.related_identifiers:
         for identifier in metadata.related_identifiers:
-            if py_.get(identifier, "relation_type.id") == "ispartof" and identifier.get(
+            if dig(identifier, "relation_type.id") == "ispartof" and identifier.get(
                 "identifier", ""
             ).startswith(f"https://{host}/api/communities/"):
                 slug = identifier.get("identifier").split("/")[5]
@@ -550,11 +565,11 @@ def search_by_doi(doi, host, token) -> Optional[str]:
         )
         response.raise_for_status()
         data = response.json()
-        if py_.get(data, "hits.total", 0) > 0:
-            return py_.get(data, "hits.hits.0.id")
+        if dig(data, "hits.total", 0) > 0:
+            return dig(data, "hits.hits.0.id")
         return None
     except requests.exceptions.RequestException as e:
-        logger.error(f"Error searching for DOI {doi}: {str(e)}", exc_info=True)
+        log.error(f"Error searching for DOI {doi}: {str(e)}", exc_info=True)
         return None
 
 
@@ -572,7 +587,7 @@ def create_draft_record(record, host, token, output):
             record["status"] = "failed_rate_limited"
             return record
         if response.status_code != 201:
-            logger.error(
+            log.error(
                 f"Failed to create draft record: {response.status_code} - {response.json()}"
             )
             record["status"] = "failed_create_draft"
@@ -584,11 +599,35 @@ def create_draft_record(record, host, token, output):
         record["status"] = "draft"
         return record
     except requests.exceptions.RequestException as e:
-        logger.error(f"Error creating draft record: {str(e)}", exc_info=True)
+        log.error(f"Error creating draft record: {str(e)}", exc_info=True)
         record["status"] = "error_draft"
         return record
 
 
+def reserve_doi(record, host, token) -> dict:
+    """Reserve a DOI for a draft record."""
+    headers = {
+        "Authorization": f"Bearer {token}",
+        "Content-Type": "application/json",
+    }
+    try:
+        response = requests.post(
+            f"https://{host}/api/records/{record.get('id')}/draft/pids/doi",
+            headers=headers,
+        )
+        response.raise_for_status()
+        data = response.json()
+        record["doi"] = data.get("doi", None)
+        record["status"] = "doi_reserved"
+        return record
+    except requests.exceptions.RequestException as e:
+        log.error(
+            f"Error reserving DOI for record {record['id']}: {str(e)}", exc_info=True
+        )
+        record["status"] = "error_reserve_doi"
+        return record
+
+
 def edit_published_record(record, host, token):
     """Create a draft from a published record in InvenioRDM"""
     headers = {
@@ -601,11 +640,12 @@ def edit_published_record(record, host, token):
         )
         response.raise_for_status()
         data = response.json()
+        print("Edited published record:", dig(data, "parent.pids.doi.identifier"))
         record["updated"] = data.get("updated", None)
         record["status"] = "edited"
         return record
     except requests.exceptions.RequestException as e:
-        logger.error(
+        log.error(
             f"Error creating draft from published record: {str(e)}", exc_info=True
         )
         record["status"] = "error_edit_published_record"
@@ -626,11 +666,12 @@ def update_draft_record(record, host, token, inveniordm_data):
         )
         response.raise_for_status()
         data = response.json()
+        print("Update draft record:", dig(data, "parent.pids.doi.identifier"))
         record["updated"] = data.get("updated", None)
         record["status"] = "updated"
         return record
     except requests.exceptions.RequestException as e:
-        logger.error(f"Error updating draft record: {str(e)}", exc_info=True)
+        log.error(f"Error updating draft record: {str(e)}", exc_info=True)
         record["status"] = "error_update_draft_record"
         return record
 
@@ -653,7 +694,7 @@ def publish_draft_record(record, host, token):
             record["status"] = "failed_rate_limited"
             return record
         if response.status_code != 202:
-            logger.error(
+            log.error(
                 f"Failed to publish draft record: {response.status_code} - {response.json()}"
             )
             record["status"] = "error_publish_draft_record"
@@ -664,7 +705,7 @@ def publish_draft_record(record, host, token):
         record["status"] = "published"
         return record
     except requests.exceptions.RequestException as e:
-        logger.error(f"Error publishing draft record: {str(e)}", exc_info=True)
+        log.error(f"Error publishing draft record: {str(e)}", exc_info=True)
         record["status"] = "error_publish_draft_record"
         return record
 
@@ -682,11 +723,11 @@ def get_record_communities(record, host, token):
         )
         response.raise_for_status()
         data = response.json()
-        if py_.get(data, "hits.total", 0) > 0:
-            return py_.get(data, "hits.hits")
+        if dig(data, "hits.total", 0) > 0:
+            return dig(data, "hits.hits")
         return None
     except requests.exceptions.RequestException as e:
-        logger.error(f"Error getting communities: {str(e)}", exc_info=True)
+        log.error(f"Error getting communities: {str(e)}", exc_info=True)
         return None
 
 
@@ -706,7 +747,7 @@ def add_record_to_community(record, host, token, community_id):
         response.raise_for_status()
         return record
     except requests.exceptions.RequestException as e:
-        logger.error(f"Error adding record to community: {str(e)}", exc_info=True)
+        log.error(f"Error adding record to community: {str(e)}", exc_info=True)
         return record
 
 
@@ -724,17 +765,23 @@ def update_legacy_record(record, legacy_key: str, field: str = None) -> dict:
         if field == "rid" and record.get("id", None) is not None:
             output = {
                 "rid": record.get("id"),
+                "doi": record.get("doi", None),
                 "indexed_at": now,
                 "indexed": "true",
                 "archived": "true",
+                "registered": "false",
             }
         elif record.get("doi", None) is not None:
             output = {
-                "doi": record.get("doi"),
-                "indexed_at": now,
-                "indexed": "true",
-                "archived": "true",
+                "registered": "true",
             }
+            # output = {
+            #     "doi": record.get("doi"),
+            #     "indexed_at": now,
+            #     "indexed": "true",
+            #     "archived": "true",
+            #     "registered": "false",
+            # }
         else:
             print(f"nothing to update for id {record.get('uuid')}")
             return record  # nothing to update
@@ -754,7 +801,7 @@ def update_legacy_record(record, legacy_key: str, field: str = None) -> dict:
         return record
 
     except requests.exceptions.RequestException as e:
-        logger.error(f"Error updating legacy record: {str(e)}", exc_info=True)
+        log.error(f"Error updating legacy record: {str(e)}", exc_info=True)
         record["status"] = "error_update_legacy_record"
         return record
 
@@ -772,11 +819,11 @@ def search_by_slug(slug: str, type: str, host: str, token: str) -> Optional[str]
         )
         response.raise_for_status()
         data = response.json()
-        if py_.get(data, "hits.total", 0) > 0:
-            return py_.get(data, "hits.hits.0.id")
+        if dig(data, "hits.total", 0) > 0:
+            return dig(data, "hits.hits.0.id")
         return None
     except requests.exceptions.RequestException as e:
-        logger.error(f"Error searching for community: {str(e)}", exc_info=True)
+        log.error(f"Error searching for community: {str(e)}", exc_info=True)
         return None
 
 
